#!/usr/bin/env python
# -*- coding: utf-8 -*-

import random
import unittest

from mocksey import generate_mock  # SUT

TEETH_PROP = "%d wombat teeth are full of ouch" % random.randint(0, 30)


class TestDummy(unittest.TestCase):
    def nop(self):
        pass


assert_equals = TestDummy().assertEqual


class WileyWombat:
    teeth = TEETH_PROP

    def masticate(self):
        while True:
            pass

    def patty_pat(self, patting_device):
        while True:
            print("blowing up your tests now, kthxbie")

    def perform(self, **feat):
        while True:
            print("Now {}-ifying {}".format(feat.items()[0]))

    def move(self, *args, **kwargs):
        while True:
            pass


class WombatMob:
    def __init__(self, a_wombat):
        self.wombat = a_wombat

    def feed(self):
        return self.wombat.masticate()

    def pat(self, patting_device="face"):
        return self.wombat.patty_pat(patting_device)

    def perform(self, **feat):
        return self.wombat.perform(**feat)

    def do_yer_business(self):
        return self.wombat.defecate()


class SimpleMockTestCase(unittest.TestCase):
    def setUp(self):
        self.mock = generate_mock(WileyWombat)
        self.mob = WombatMob(self.mock)

    def test_mock_expect_once(self):
        """mocksey.MockseyObject: Mock blows up if an expected function is not called"""
        self.mock.expect_once("masticate")
        try:
            self.mock.run_asserts(assert_equals)
            # Can't rely on self.fail, as that's an AssertionError
            raise Exception("Did not blow up when an expected function was not called")
        except AssertionError:
            pass  # we want it to kablooey this way, hooray!

    def test_mock_expect_multiple(self):
        """mocksey.MockseyObject: Mock blows up if an expected function is not called"""
        call_count = random.randint(1, 50)
        self.mock.expect_call_count("masticate", call_count)
        for waffle in range(call_count - 1):
            self.mock.masticate()
        try:
            self.mock.run_asserts(assert_equals)
            # Can't rely on self.fail, as that's an AssertionError
            raise Exception("Did not blow up when an expected function was not called enough")
        except AssertionError:
            pass  # we want it to kablooey this way, hooray!

        self.mock.masticate()
        self.mock.run_asserts()

    def test_mock_expect_never(self):
        """mocksey.MockseyObject: Mock blows up if an function who is supposed to never be called is called"""
        self.mock.expect_never("go_crazygonuts")
        self.mock.go_crazygonuts()

        try:
            self.mock.run_asserts(assert_equals)
            # Can't rely on self.fail, as that's an AssertionError
            raise Exception("Did not blow up when a function that we expected to never be called was called")
        except AssertionError:
            pass  # we want it to kablooey this way, hooray!

    ############################# ARG VERIFICATION ##################################

    def test_mock_validates_args_for_single_call(self):
        """mocksey.MockseyObject.run_asserts: run_asserts properly complains when single call has wrong args"""

        self.mock.expect_once("move", args=("north", "south"))
        self.mock.move("nerth", "serth")
        try:
            self.mock.run_asserts()
            raise Exception("Did not blow up when an expected function was called with unexpected args")
        except AssertionError:
            pass

    def test_mock_validates_any_args_for_single_wildcard_call(self):
        """mocksey.MockseyObject.run_asserts: run_asserts is happy with any args when args aren't specified"""

        self.mock.expect_once("move")
        self.mock.move("nerth", "serth")
        self.mock.run_asserts()

    def test_mock_validates_args_for_multiple_calls(self):
        """
        mocksey.MockseyObject.run_asserts: run_asserts properly complains when one or all of multiple calls has wrong
        args
        """
        self.mock.expect_at("move", 2, args=("east", "butter"))
        self.mock.expect_at("move", 0, args=("north", "south"))
        self.mock.move("north", "south")
        self.mock.move("north", "crooked")
        self.mock.move("yeast", "buttermilk")
        try:
            self.mock.run_asserts()
            raise Exception("Did not blow up when an expected function was called with unexpected args")
        except AssertionError:
            pass

    ############################# KWARG VERIFICATION ##################################

    def test_mock_validates_kwargs_for_single_call(self):
        """mocksey.MockseyObject.run_asserts: run_asserts properly complains when call has wrong kwargs"""

        self.mock.expect_once("move", kwargs={"north": 600, "south": 900})
        self.mock.move(north=900, south=500, dance_style="chicken-headed two-step")
        try:
            self.mock.run_asserts()
            raise Exception("Did not blow up when an expected function was called with unexpected kwargs")
        except AssertionError:
            pass

    def test_mock_validates_any_kwargs_for_single_wildcard_call(self):
        """mocksey.MockseyObject.run_asserts: run_asserts is happy with any kwargs when args aren't specified"""

        self.mock.expect_once("move")
        self.mock.move(north=540, south=690)
        self.mock.run_asserts()

    def test_mock_validates_kwargs_for_multiple_calls(self):
        """
        mocksey.MockseyObject.run_asserts: run_asserts properly complains when one or all of multiple calls has wrong
        kwargs
        """
        self.mock.expect_at("move", 2, kwargs={"north": 600, "south": 900})
        self.mock.expect_at("move", 0, kwargs={"north": 554, "south": 224})

        self.mock.move(north=554, south=224)
        self.mock.move(north=424, south=585)
        self.mock.move(north=540, south=234)
        try:
            self.mock.run_asserts()
            raise Exception("Did not blow up when an expected function was called with unexpected kwargs")
        except AssertionError:
            pass


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import random
import unittest

from mocksey import MockseyObject, generate_mock  # SUT

TEETH_PROP = "%d wombat teeth are full of ouch" % random.randint(0, 30)


class WileyWombat:
    teeth = TEETH_PROP

    def masticate(self):
        while True:
            pass

    def patty_pat(self, patting_device):
        while True:
            print("blowing up your tests now, kthxbie")

    def perform(self, **feat):
        while True:
            print("Now {}-ifying {}".format(feat.items()[0]))


class WombatMob:
    def __init__(self, a_wombat):
        self.wombat = a_wombat

    def feed(self):
        return self.wombat.masticate()

    def pat(self, patting_device="face"):
        return self.wombat.patty_pat(patting_device)

    def perform(self, **feat):
        return self.wombat.perform(**feat)

    def do_yer_business(self):
        return self.wombat.defecate()


class SimpleMockTestCase(unittest.TestCase):
    def setUp(self):
        self.mock = generate_mock(WileyWombat)
        self.mob = WombatMob(self.mock)

    def test_mock_steals_class_name(self):
        """mocksey.generate_mock: Mock "inherits" mocked class name"""
        self.assertEqual("WileyWombat", self.mock.__class__.__name__, "Mock did not inherit class name")

    def test_mock_reports_itself_as_mock(self):
        """mocksey.generate_mock: Mock's repr reveals it's true nature"""
        self.assertEqual("MockWileyWombat", "{}".format(self.mock), "Mock did not identify as mock in repr")

    def test_mock_inherits_attributes(self):
        """mocksey.generate_mock: Attributes on mocked class get mirrored onto mock"""
        self.assertTrue(hasattr(self.mock, "teeth"), "Mock does not have an attribute that the mocked object did")
        self.assertEqual(TEETH_PROP, self.mock.teeth, "Mock's value was not set correctly.")

    def test_mock_inherits_callable(self):
        """mocksey.generate_mock: Methods on mocked class are mirrored and emptied"""
        self.assertTrue(hasattr(self.mock, "masticate"), "Mock does not have a method that the mocked object did")
        self.mock.masticate()

    def test_mock_returns(self):
        """mocksey.MockseyObject: Mocked function returns requested value"""
        masticate_return = "Okay okay, I'll chew it up!"
        self.mock.returns("masticate", masticate_return)
        self.assertEqual(masticate_return, self.mob.feed())

    def test_mock_returns_at(self):
        """mocksey.MockseyObject: Mocked function returns requested value at requested index"""
        masticate_return = "Okay okay, I'll chew it up attempt %d!"
        for trial in range(random.randint(2, 15)):
            self.mock.returns_at(trial, "masticate", masticate_return % (trial))
            self.assertEqual(masticate_return % (trial), self.mob.feed())

    def test_unexpected_calls_are_tracked(self):
        """mocksey.MockseyObject: Mocksey keeps track of method calls even when they're not being asserted"""
        call_count = random.randint(1, 50)
        for call_time in range(call_count):
            self.mob.pat()

        self.assertEqual(call_count, self.mock.called_functions["patty_pat"]["count"])

    def test_unexpected_calls_track_args(self):
        """mocksey.MockseyObject: Mocksey keeps track of method call args even when they're not being asserted"""
        pattable_patting_parts = ["Hand", "cheek", "foot", "toe", "30-foot pole"]
        choice = random.choice(pattable_patting_parts)
        self.mob.pat(choice)
        self.assertEqual((choice,), self.mock.called_functions["patty_pat"][0]["args"])

    def test_unexpected_calls_track_kwargs(self):
        """mocksey.MockseyObject: Mocksey keeps track of method call kwargs even when they're not being asserted"""
        amazing_wombat_actions = [
            {"fly": "faster than light"},
            {"dig": "like a boss"},
            {"trample": "armies of zombies"},
            {"shoot": "Space-mutants with eye lazorz"},
            {"speak": "perfect Russian"},
            {"dance": "all teh koalas"},
            {"be": "cute...or says my wife", "scare": "all the crawdillys out"},
            {"lead": "super duper secret life of crime, villany and wombatery"},
        ]
        choice = random.choice(amazing_wombat_actions)
        self.mob.perform(**choice)

        self.assertEqual(choice, self.mock.called_functions["perform"][0]["kwargs"])

    def test_raises(self):
        """mocksey.MockseyObject: Mocksey can raise on a given function call"""

        class ZombieInvasion(Exception):
            pass

        self.mock.raises("defecate", ZombieInvasion)

        try:
            self.mob.do_yer_business()
            self.fail("Did not raise as expected")
        except ZombieInvasion:
            pass  # Just as I had feared!  All the Safety Wombats will be constipated during the Zombie Invasion

    def test_supports_with_statement(self):
        """mocksey.MockseyObject: Mocksey mocks can be used in python's 'with' statement"""
        with MockseyObject() as mocksey:
            mocksey.expect_once("something")


# test module mocking...i.e. import something like smtp and set a mock on smtp.SMTP

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
from mocksey import mocksey_assert_equal


def tweak_mock(mock):
    call = mock.call

    def mocksey_assert_called_once_with(self, *args, **kwargs):
        """assert that the mock was called exactly once and with the specified
        arguments.

        Mocksey copied this from voidspace's great "mock" and modified it for you:
          http://www.voidspace.org.uk/python/mock/
          That License:
            Copyright (c) 2003-2010, Michael Foord
            All rights reserved.
            E-mail : fuzzyman AT voidspace DOT org DOT uk
        """
        if not self.call_count == 1:
            raise AssertionError(
                f"Mock '{self}' expected to be called once."
                f" Called {self.call_count:d} times with {self.call_args_list}."
            )
        return self.assert_called_with(*args, **kwargs)

    def mocksey_assert_any_call(self, *args, **kwargs):
        """assert the mock has been called with the specified arguments.

        The assert passes if the mock has *ever* been called, unlike
        `assert_called_with` and `assert_called_once_with` that only pass if
        the call is the most recent one.

        Mocksey copied this from voidspace's great "mock" and modified it for you:
          http://www.voidspace.org.uk/python/mock/
          That License:
            Copyright (c) 2003-2010, Michael Foord
            All rights reserved.
            E-mail : fuzzyman AT voidspace DOT org DOT uk
        """
        if self.call_count <= 1:
            self.assert_called_with(*args, **kwargs)
        else:
            kall = call(*args, **kwargs)
            if kall not in self.call_args_list:
                expected_string = self._format_mock_call_signature(args, kwargs)
                foo = self.call_args_list
                if self._mock_name:
                    foo = repr(self.call_args_list).replace("call(", self._mock_name + "(")
                raise AssertionError(f"{expected_string} call not found among:\n\t{foo}")

    def mocksey_assert_called_with(self, *args, **kwargs):
        """assert that the mock was called with the specified arguments.

        Raises an AssertionError if the args and keyword args passed in are
        different to the last call to the mock.

        Mocksey copied this from voidspace's great "mock" and modified it for you:
          http://www.voidspace.org.uk/python/mock/
          That License:
            Copyright (c) 2003-2010, Michael Foord
            All rights reserved.
            E-mail : fuzzyman AT voidspace DOT org DOT uk
        """
        if not self.called:
            raise AssertionError("Mock '{}' was never called.".format(self))

        nofail = "Nothing!"

        arg_out = nofail
        kwarg_out = nofail
        try:
            mocksey_assert_equal(args, self.call_args[0])
        except AssertionError as e:
            arg_out = str(e)

        try:
            mocksey_assert_equal(kwargs, self.call_args[1])
        except AssertionError as e:
            kwarg_out = str(e)

        if arg_out != nofail or kwarg_out != nofail:
            output = (
                f"{self} Suffered the following call issues (expected != actual):"
                f"\nArgs:  {arg_out}\nKwargs: {kwarg_out}"
            )
            raise AssertionError(output)

    mock.NonCallableMock.assert_called_once_with = mocksey_assert_called_once_with
    mock.NonCallableMock.assert_any_call = mocksey_assert_any_call
    mock.NonCallableMock.assert_called_with = mocksey_assert_called_with

    return mock


if __name__ == "__main__":
    from unittest import mock

    tweak_mock(mock)
    foo = mock.MagicMock(name="past")
    foo("lollipop", rambo="doody")
    # foo('aosenuth', 'asoneuht2aou')
    foo.assert_called_once_with("aosenuth", "asoneuht")
    # foo.assert_called_with('lollipoop', rambo='duty', zippo=True)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mocksey', 'mocksey.tests']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'mocksey',
    'version': '0.4.0',
    'description': 'Stupidly-simple python mocking utility with moxie.',
    'long_description': "mocksey\n=======\n\nStupidly-simple python mocking utility with moxie.\n\n|BuildImage|_\n\n.. image:: https://pypip.in/v/mocksey/badge.png\n    :target: https://crate.io/packages/mocksey/\n    :alt: Latest PyPI version\n\n.. image:: https://pypip.in/d/mocksey/badge.png\n    :target: https://crate.io/packages/mocksey/\n    :alt: Number of PyPI downloads\n\n|Coverage Status|\n\n .. |Coverage Status| image:: https://coveralls.io/repos/mitgr81/mocksey/badge.png\n    :target: https://coveralls.io/r/mitgr81/mocksey\n\n\nMocksey Motivation\n==================\n\nI was teaching a class on unit testing to a group of co-workers who were familiar with `Simple Test for PHP <http://www.simpletest.org/>`_ so I hacked together what is becoming Mocksey.\n\nMocksey the TDD'd version of that TDD utilty. `It's so meta even this acronym <http://xkcd.com/917/>`_.\n\nInstallation\n============\n\nEither find mocksey on PyPI_ or install it with pip or easy_install\n::\n\n  pip install mocksey\n  #or\n  easy_install mocksey\n\nBasic Usage\n===========\n\nIt's pretty simple.  Create a mocked object with generate_mock, inject it (or monkey patch) and set up your assertions.  After your function call(s), simply call 'run_asserts' and win!\n\nThe unit tests are a pretty decent working example.\n\nTweaksey\n========\n\nTweaksey is a collection of beautification wrappers.  Currently there's only one around mock_, but there may be more in the future.  It'll look best if you also have nose_ installed, and may only be worth it in that case, actually.  Anyhow, to use it simply import tweaksey from mocksey and get your copy of the mock package from ``tweaksey.tweak_mock``.  Your mock assertions should now have a touch more friendliness.  If there are more you'd like to add, go for it!  Michael Foord, if you want to take the output and run, that cool too (conversely, if you don't like that I did this, I'll kill it square dead).\n\n\nChangelog\n=========\n\n0.3.1\n-----\nMocksey now sets nose to full diff mode.\n\n0.3.0\n-----\nTweaksey now requires you to pass in the 'mock' library that you're\ntweaking.  This allows one to apply mocks to python3's ``unittest.mock``.\n\n\nLicense\n=======\nThis software is hereby released under the MIT License, as seen in the LICENSE file\n\n.. |BuildImage| image:: https://secure.travis-ci.org/mitgr81/mocksey.png\n.. _BuildImage: https://travis-ci.org/mitgr81/mocksey\n.. _PyPI: http://pypi.python.org/pypi/mocksey\n.. _mock: http://www.voidspace.org.uk/python/mock/\n.. _nose: https://pypi.python.org/pypi/nose/\n",
    'author': 'Chris McGraw',
    'author_email': 'mitgr81+mocksey@mitgr81.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/mitgr81/mocksey',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest
from unittest import mock

from mocksey import tweaksey  # SUT


class TweakseyTweaksMockCase(unittest.TestCase):
    def setUp(self):
        self.ncm_acow = mock.NonCallableMock.assert_called_once_with
        self.ncm_acw = mock.NonCallableMock.assert_called_with
        self.ncm_aac = mock.NonCallableMock.assert_any_call
        tweaksey.tweak_mock(mock)

        self.some_mock = mock.MagicMock()

    def tearDown(self):
        mock.NonCallableMock.assert_called_with = self.ncm_acw
        mock.NonCallableMock.assert_called_once_with = self.ncm_acow
        mock.NonCallableMock.assert_any_call = self.ncm_aac

    def test_monkey_patching(self):
        """mocksey.tweaksey.tweak_mock: tweaksey monkey patches / duck punches mock's methods"""
        self.assertIn("tweaksey", str(mock.NonCallableMock.assert_any_call.__code__))
        self.assertIn("tweaksey", str(mock.NonCallableMock.assert_called_once_with.__code__))
        self.assertIn("tweaksey", str(mock.NonCallableMock.assert_called_with.__code__))

    def test_assert_called_once_with_override_assert_any_message_with_bad_arg(self):
        """mocksey.tweaksey.tweak_mock.assert_called_once_with: Override message for arg match failure"""
        self.some_mock()
        with self.assertRaises(AssertionError) as con_man:
            self.some_mock.assert_called_once_with("pants")

        actual = str(con_man.exception)
        expected = "Suffered the following call issues (expected != actual):\n"
        self.assertIn(expected, actual)
        expected = "Args:  Tuples differ: ('pants',) != ()\n\nFirst tuple contains 1 additional elements.\nFirst extra element 0:\n'pants'\n\n- ('pants',)\n+ ()\n"
        self.assertIn(expected, actual)
        expected = "Kwargs: Nothing!"
        self.assertIn(expected, actual)

    def test_assert_called_once_with_override_assert_any_message_with_bad_kwarg(self):
        """mocksey.tweaksey.tweak_mock.assert_called_once_with: Override message for kwarg match failure"""
        self.some_mock()
        with self.assertRaises(AssertionError) as con_man:
            self.some_mock.assert_called_once_with(clothing="pants")

        actual = str(con_man.exception)
        expected = "Suffered the following call issues (expected != actual):\n"
        self.assertIn(expected, actual)
        expected = "Args:  Nothing!\n"
        self.assertIn(expected, actual)
        expected = "Kwargs: {'clothing': 'pants'} != {}\n- {'clothing': 'pants'}\n+ {}"
        self.assertIn(expected, actual)

    def test_assert_called_once_with_dont_interfere_when_call_was_good(self):
        """mocksey.tweaksey.tweak_mock.assert_called_once_with: Don't raise without a failure"""
        self.some_mock("pants")
        try:
            self.some_mock.assert_called_once_with("pants")  # SUT
        except AssertionError:
            self.fail("I know this test looks funny, but I need to make sure we don't accidentally raise")

    def test_assert_called_once_with_missing_call(self):
        """mocksey.tweaksey.tweak_mock.assert_called_once_with: Don't raise without a failure"""
        self.some_mock.__str__.return_value = "lalalalala"

        with self.assertRaises(AssertionError) as con_man:
            self.some_mock.assert_called_once_with(clothing="pants")

        actual = str(con_man.exception)
        expected = "Mock 'lalalalala' expected to be called once. Called 0 times with []."
        self.assertIn(expected, actual)

    #######################################

    def test_assert_called_with_override_assert_any_message_with_bad_arg(self):
        """mocksey.tweaksey.tweak_mock.assert_called_with: Override message for arg match failure"""
        self.some_mock()
        with self.assertRaises(AssertionError) as con_man:
            self.some_mock.assert_called_with("pants")

        actual = str(con_man.exception)
        expected = "Suffered the following call issues (expected != actual):\n"
        self.assertIn(expected, actual)
        expected = (
            "Args:  Tuples differ: ('pants',) != ()\n\nFirst tuple contains 1 additional elements."
            "\nFirst extra element 0:\n'pants'\n\n- ('pants',)\n+ ()\n"
        )
        self.assertIn(expected, actual)
        expected = "Kwargs: Nothing!"
        self.assertIn(expected, actual)

    def test_assert_called_with_override_assert_any_message_with_bad_kwarg(self):
        """mocksey.tweaksey.tweak_mock.assert_called_with: Override message for kwarg match failure"""
        self.some_mock()
        with self.assertRaises(AssertionError) as con_man:
            self.some_mock.assert_called_with(clothing="pants")

        actual = str(con_man.exception)
        expected = "Suffered the following call issues (expected != actual):\n"
        self.assertIn(expected, actual)
        expected = "Args:  Nothing!\n"
        self.assertIn(expected, actual)
        expected = "Kwargs: {'clothing': 'pants'} != {}\n- {'clothing': 'pants'}\n+ {}"
        self.assertIn(expected, actual)

    def test_assert_called_with_dont_interfere_when_call_was_good(self):
        """mocksey.tweaksey.tweak_mock.assert_called_with: Don't raise without a failure"""
        self.some_mock("pants")
        try:
            self.some_mock.assert_called_with("pants")  # SUT
        except AssertionError:
            self.fail("I know this test looks funny, but I need to make sure we don't accidentally raise")

    def test_assert_called_with_missing_call(self):
        """mocksey.tweaksey.tweak_mock.assert_called_with: Don't raise without a failure"""
        self.some_mock.__str__.return_value = "dadadada"

        with self.assertRaises(AssertionError) as con_man:
            self.some_mock.assert_called_with(clothing="pants")

        actual = str(con_man.exception)
        expected = "Mock 'dadadada' was never called."
        self.assertEqual(expected, actual)

    #######################################

    # test single calls fall through
    def test_assert_any_call_override_assert_any_message_with_bad_arg(self):
        """mocksey.tweaksey.tweak_mock.assert_any_call: Override message for arg match failure"""
        self.some_mock()
        with self.assertRaises(AssertionError) as con_man:
            self.some_mock.assert_any_call("pants")

        actual = str(con_man.exception)
        expected = "Suffered the following call issues (expected != actual):\n"
        self.assertIn(expected, actual)
        expected = (
            "Args:  Tuples differ: ('pants',) != ()\n\nFirst tuple contains 1 additional elements."
            "\nFirst extra element 0:\npants\n\n- ('pants',)\n+ ()\n"
        )
        self.assertIn(expected, actual)
        expected = "Kwargs: Nothing!"
        self.assertIn(expected, actual)

    def test_assert_any_call_override_assert_any_message_with_bad_kwarg(self):
        """mocksey.tweaksey.tweak_mock.assert_any_call: Override message for kwarg match failure"""
        self.some_mock()
        with self.assertRaises(AssertionError) as con_man:
            self.some_mock.assert_any_call(clothing="pants")

        actual = str(con_man.exception)
        expected = "Suffered the following call issues (expected != actual):\n"
        self.assertIn(expected, actual)
        expected = "Args:  Nothing!\n"
        self.assertIn(expected, actual)
        expected = "Kwargs: {'clothing': 'pants'} != {}\n- {'clothing': 'pants'}\n+ {}"
        self.assertIn(expected, actual)

    # test multiple calls
    def test_assert_any_call_override_assert_any_message_with_bad_arg(self):
        """mocksey.tweaksey.tweak_mock.assert_any_call: Override message for arg match failure"""
        self.some_mock()
        self.some_mock("yes again")
        with self.assertRaises(AssertionError) as con_man:
            self.some_mock.assert_any_call("pants")

        actual = str(con_man.exception)
        expected = "mock('pants') call not found among:\n\t[call(), call('yes again')]"
        self.assertEqual(expected, actual)

    def test_assert_any_call_override_assert_any_message_with_bad_kwarg(self):
        """mocksey.tweaksey.tweak_mock.assert_any_call: Override message for kwarg match failure"""
        self.some_mock()
        self.some_mock(and_again=True)
        self.some_mock("once", "more")
        with self.assertRaises(AssertionError) as con_man:
            self.some_mock.assert_any_call(clothing="pants")

        actual = str(con_man.exception)
        expected = (
            "mock(clothing='pants') call not found among:\n\t[call(), call(and_again=True), call('once', 'more')]"
        )
        self.assertEqual(expected, actual)

    # test multiple calls with named mocks
    def test_assert_any_call_override_assert_any_message_with_bad_arg(self):
        """mocksey.tweaksey.tweak_mock.assert_any_call: Override message for arg match failure"""
        local_mock = mock.MagicMock(name="the_doctor")
        local_mock()
        local_mock("yes again")
        with self.assertRaises(AssertionError) as con_man:
            local_mock.assert_any_call("pants")

        actual = str(con_man.exception)
        expected = "the_doctor('pants') call not found among:\n\t[the_doctor(), the_doctor('yes again')]"
        self.assertEqual(expected, actual)

    def test_assert_any_call_override_assert_any_message_with_bad_kwarg(self):
        """mocksey.tweaksey.tweak_mock.assert_any_call: Override message for kwarg match failure"""
        local_mock = mock.MagicMock(name="the_doctor")
        local_mock()
        local_mock(and_again=True)
        local_mock("once", "more")
        with self.assertRaises(AssertionError) as con_man:
            local_mock.assert_any_call(clothing="pants")

        actual = str(con_man.exception)
        expected = (
            "the_doctor(clothing='pants') call not found among:"
            "\n\t[the_doctor(), the_doctor(and_again=True), the_doctor('once', 'more')]"
        )
        self.assertEqual(expected, actual)

    def test_assert_any_call_dont_interfere_when_call_was_good(self):
        """mocksey.tweaksey.tweak_mock.assert_any_call: Don't raise without a failure"""
        self.some_mock("slippers")
        self.some_mock("pants")
        self.some_mock(1234)
        try:
            self.some_mock.assert_any_call("pants")  # SUT
        except AssertionError:
            self.fail("I know this test looks funny, but I need to make sure we don't accidentally raise")

    def test_assert_any_call_missing_call(self):
        """mocksey.tweaksey.tweak_mock.assert_any_call: Don't raise without a failure"""
        self.some_mock.__str__.return_value = "dadadada"

        with self.assertRaises(AssertionError) as con_man:
            self.some_mock.assert_any_call(clothing="pants")

        actual = str(con_man.exception)
        expected = "Mock 'dadadada' was never called."
        self.assertEqual(expected, actual)


if __name__ == "__main__":
    unittest.main()



import types
import unittest


def mocksey_assert_equal(expected, actual, message=None):
    class TestDummy(unittest.TestCase):
        def nop(self):
            pass

    asserter = TestDummy().assertEqual
    asserter.__self__.maxDiff = None

    asserter(expected, actual, message)


class MockseyObject:
    def __init__(self):
        self.expected_functions = {}
        self.called_functions = {}

    def __enter__(self):
        return self

    def __exit__(self, *args, **kwargs):
        pass

    def __repr__(self):
        return f"Mock{self.__class__.__name__}"

    def __getattr__(self, function_name):
        if function_name in self.expected_functions:

            def function(*args, **kwargs):
                self.called_functions[function_name] = self.called_functions.get(function_name, {"count": 0})
                self.called_functions[function_name]["count"] += 1
                count = self.called_functions[function_name]["count"]

                self.called_functions[function_name][count - 1] = {"args": args, "kwargs": kwargs}
                if "return" in self.expected_functions[function_name]:
                    return self.expected_functions[function_name]["return"].get(
                        count - 1,
                        self.expected_functions[function_name]["return"].get("*", None),
                    )

            return function
        else:

            def unexpected_function(*args, **kwargs):
                # We may want to add a flag to do strict checks here where it fails if this function is called
                # ...not now though
                pass

            return unexpected_function

    def run_asserts(self, assert_equal=mocksey_assert_equal):
        for function_name, function_data in self.expected_functions.items():
            msg_head = f"Mockseyed{self.__class__.__name__} expected '{function_name}'"
            actual = self.called_functions.get(function_name, {"count": 0})["count"]
            if function_data.get("count", 0) > 0:
                expected = self.expected_functions[function_name]["count"]
                assert_equal(expected, actual, f"{msg_head} to be called {expected} times, but it was called {actual}.")
                for call in range(function_data["count"]):
                    expected_params = function_data.get(call, {"args": "*", "kwargs": "*"})
                    if expected_params["args"] != "*":
                        assert_equal(expected_params["args"], self.called_functions[function_name][call]["args"])
                    if expected_params["kwargs"] != "*":
                        assert_equal(expected_params["kwargs"], self.called_functions[function_name][call]["kwargs"])
            elif function_data.get("count", 0) == -1:
                assert_equal(0, actual, f"{msg_head} to never be called times, but it was called {actual}.")

    def expect_once(self, expected_function, args="*", kwargs="*"):
        self.expect_call_count(expected_function, 1)
        self.expect_at(expected_function, 0, args=args, kwargs=kwargs)

    def expect_at(self, expected_function, index, args="*", kwargs="*"):
        self.expected_functions[expected_function][index] = {"args": args, "kwargs": kwargs}
        self.expect_call_count(expected_function, index + 1)

    def expect_call_count(self, expected_function, count):
        self.expected_functions[expected_function] = self.expected_functions.get(expected_function, {"count": 0})
        if count == -1 or self.expected_functions[expected_function].get("count", 0) < count:
            self.expected_functions[expected_function]["count"] = count

    def expect_never(self, expected_function):
        self.expect_call_count(expected_function, -1)

    def returns(self, returning_function, return_value):
        self.returns_at("*", returning_function, return_value)

    def returns_at(self, count, returing_function, retval):
        self.expected_functions[returing_function] = self.expected_functions.get(returing_function, {"count": 0})
        self.expected_functions[returing_function]["return"][count] = retval

    def raises(self, raising_function, exception):
        def mock_raiser(*args, **kwargs):
            raise exception

        setattr(self, raising_function, mock_raiser)
        return mock_raiser


def generate_mock(subject):
    mock = MockseyObject()
    mock.__class__.__name__ = subject.__name__
    for attr in dir(subject):
        if len(attr) > 1 and attr[1] != "_":
            val = getattr(subject, attr)
            if callable(val):
                mock.expected_functions[attr] = {"return": {"*": None}}
                types.MethodType(val, mock)
            else:
                setattr(mock, attr, val)
    return mock
